/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.news;

import javax.annotation.Nonnull;
import java.io.IOException;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.semantic.rss.RssFeed;
import it.tidalwave.semantic.document.Document;

/***********************************************************************************************************************
 *
 * FIXME: rename to NewsService
 * 
 * @stereotype Service
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface NewsService
  {
    public static interface NewsFeedCallback
      {
        /***************************************************************************************************************
         *
         * Notifies that there's a news feed available.
         * 
         * @param newsFeed  the news feed
         *
         **************************************************************************************************************/
        @ThreadConfined(type=ThreadType.ANY)    
        public void notifyFeedAvailable (@Nonnull RssFeed newsFeed);

        /***************************************************************************************************************
         *
         * Notifies that there's an obsolete news feed available.
         * 
         * @param newsFeed  the news feed
         *
         **************************************************************************************************************/
        @ThreadConfined(type=ThreadType.ANY)    
        public void notifyCachedFeedAvailable(RssFeed rssFeed);
        
        /***************************************************************************************************************
         *
         * Notifies that there's no news feed available.
         *
         **************************************************************************************************************/
        @ThreadConfined(type=ThreadType.ANY)
        public void notifyFeedUnavailable();

        /***************************************************************************************************************
         *
         * 
         *
         **************************************************************************************************************/
        @ThreadConfined(type=ThreadType.ANY)
        public void notifyFeedCouldBeDownloaded();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void start();

    /*******************************************************************************************************************
     *
     * Gets the News feed. This method is asynchronous (non blocking) and communicates by means of the callback. It 
     * might notify that the News feed is available, or unavailable, or it might be downloaded if desired. If 
     * networking connections are enabled, the News feed is downloaded when needed.
     * 
     * @param  callBack  the callback
     *
     ******************************************************************************************************************/
    @Nonnull @ThreadConfined(type=ThreadType.ANY)
    public void getNewsFeed (@Nonnull NewsFeedCallback callBack);

    /*******************************************************************************************************************
     *
     * Always downloads a News feed.
     * 
     * @param  callBack  the callback
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void getNewsFeedAlways (@Nonnull NewsFeedCallback callBack);
    
    // FIXME: drop all methods below, expose as Readable role for Document and RssFeed.
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void markRead (@Nonnull Document message)
      throws IOException;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void markNotRead (@Nonnull Document message)
      throws IOException;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public boolean isRead (@Nonnull Document message);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void markAllRead (@Nonnull RssFeed newsFeed)
      throws IOException;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public boolean isAllRead (@Nonnull RssFeed newsFeed);    
  }
