/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.news.impl;

import java.util.List;
import java.util.ArrayList;
import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.util.Set;
import java.util.TreeSet;
import java.io.File;
import java.io.IOException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.mobile.io.FileSystem;
import it.tidalwave.mobile.io.MasterFileSystem;
import it.tidalwave.mobile.io.StringsIO;
import it.tidalwave.semantic.document.Document;
import it.tidalwave.semantic.rss.RssFeed;
import static it.tidalwave.semantic.rss.RssVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ReadMessagesLog 
  {
    private static final Logger log = LoggerFactory.getLogger(ReadMessagesLog.class);

    @Nonnull
    /* package */ final File readLogFile;

    /* package */ final Set<Id> readMessages = new TreeSet<Id>();
    
    @Nonnull
    private final Provider<MasterFileSystem> masterFileSystem = Locator.createProviderFor(MasterFileSystem.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ReadMessagesLog() 
      throws IOException 
      {
        final FileSystem internalFileSystem = masterFileSystem.get().getInternalFileSystem();
        readLogFile = internalFileSystem.getFile("blog-read-messages.txt");
        load();
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void markRead (final @Nonnull Document message)
      throws IOException
      {
        log.info("markRead({})", message);
        readMessages.add(message.getId());
        store();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void markNotRead (final @Nonnull Document message)
      throws IOException
      {
        log.info("markUnread({})", message);
        readMessages.remove(message.getId());
        store();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void markAllRead (final @Nonnull RssFeed rssFeed)
      throws IOException
      {
        log.info("markAllRead({})", rssFeed);

        try
          {
            for (final Document message : rssFeed.get(MESSAGES))
              {
                readMessages.add(message.getId());
              }

            store();
          }
        catch (NotFoundException e)
          {
            log.warn("markAllRead()", e);
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public boolean isRead (final @Nonnull Document message)
      {
        return readMessages.contains(message.getId());
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isAllRead (final @Nonnull RssFeed rssFeed)
      {
        try
          {
            final List<Id> messageIds = new ArrayList<Id>();

            for (final Document message : rssFeed.get(MESSAGES))
              {
                messageIds.add(message.getId());
              }

            messageIds.removeAll(readMessages);

            log.info(">>>> unread messages: %d", messageIds.size());

            return messageIds.isEmpty();
          }
        catch (NotFoundException e)
          {
            log.warn("isAllRead()", e);
            return false;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void load()
      throws IOException
      {
        if (readLogFile.exists())
          {
            log.trace(">>>> reading from {}", readLogFile.getAbsolutePath());
            readMessages.addAll(new StringsIO(readLogFile).load(new StringsIO.Processor<Id>()
              {
                @Nonnull
                public Id processString (final @Nonnull String string)
                  {
                    return new Id(string);
                  }
              }));
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void store()
      throws IOException
      {
        new StringsIO(readLogFile).store(readMessages);
      }    
  }
